// AplView.cpp : implementation of the CAplView class
//

#include "stdafx.h"
#include "Apl.h"

#include "AplDoc.h"
#include "AplView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CAplView

IMPLEMENT_DYNCREATE(CAplView, CListView)

BEGIN_MESSAGE_MAP(CAplView, CListView)
   //{{AFX_MSG_MAP(CAplView)
	ON_WM_CREATE()
	ON_NOTIFY_REFLECT(LVN_GETDISPINFO, OnGetDispInfo)
	ON_NOTIFY_REFLECT(LVN_ITEMCHANGED, OnItemchanged)
	ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAplView construction/destruction

CAplView::CAplView()
{
   m_bNoSelection = FALSE;
}

CAplView::~CAplView()
{
}

BOOL CAplView::PreCreateWindow(CREATESTRUCT& cs)
{
// default processing
   BOOL bReturn = CListView::PreCreateWindow(cs);

// the list is on REPORT mode by default
   cs.style |= LVS_REPORT;

// the list is single selection only
   cs.style |= LVS_SINGLESEL;

// the selected item stays in the selected color even 
// if the list view loses focus
   cs.style |= LVS_SHOWSELALWAYS;

   return bReturn;
}


/////////////////////////////////////////////////////////////////////////////
// CAplView drawing

void CAplView::OnInitialUpdate()
{
// fill the list view with the processes
   FillList();
}

// fill the list and return the number of inserted processes
int CAplView::FillList()
{
// 1. don't forget to delete the actual content of the list view
   m_ProcessIndex.RemoveAll();
   GetListCtrl().DeleteAllItems();

// count the number of process to be returned
   int iCount = 0;

// get the process list from the document
   CAplDoc* pDoc = GetDocument();
   ASSERT(pDoc != NULL);

   if (pDoc != NULL)
   {
   // 2. get process list stored into the document
      CProcessList&  ProcessList = pDoc->GetProcessList();
      CProcess*      pProcess = NULL;

   // 3. get one process after the other
      POSITION Pos = 0;
      for (
            pProcess = ProcessList.GetFirst(Pos); 
            (pProcess != NULL); 
            pProcess = ProcessList.GetNext(Pos)
          )
      {
      // add each process in the list
         if (pProcess != NULL)
         {
         // 4. insert the process into the list view
            InsertItem(pProcess, iCount);

         // 5. build the cross reference index PID --> row in the list view
            m_ProcessIndex[(LPVOID)pProcess->GetPID()] = iCount;

         // next...
            iCount++;
         }
      }

   // 6. set the selection on the first process
      if (iCount > 0)
         GetListCtrl().SetItemState(
            0, 
            LVIS_SELECTED | LVIS_FOCUSED, 
            LVIS_SELECTED | LVIS_FOCUSED
            );
   }

   return iCount;
}

/////////////////////////////////////////////////////////////////////////////
// CAplView diagnostics

#ifdef _DEBUG
void CAplView::AssertValid() const
{
   CListView::AssertValid();
}

void CAplView::Dump(CDumpContext& dc) const
{
   CListView::Dump(dc);
}

CAplDoc* CAplView::GetDocument() // non-debug version is inline
{
   ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CAplDoc)));
   return (CAplDoc*)m_pDocument;
}
#endif //_DEBUG

// handler for WM_CREATE message which is used to define the columns of the list
int CAplView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
// default MFC processing
   if (CListView::OnCreate(lpCreateStruct) == -1)
      return -1;

// get a reference to the mapped CListCtrl   
   CListCtrl& Itself = GetListCtrl();

// set the columns for the header
   LVCOLUMN Column;
   Column.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;

// Add the columns.
   Column.cx = 70;
   Column.iSubItem = COLUMN_NAME;
   Column.pszText = _T("Name");
   Column.fmt = LVCFMT_LEFT;
   Itself.InsertColumn(Column.iSubItem, &Column);

   Column.cx = 40;
   Column.iSubItem = COLUMN_PID;
   Column.pszText = _T("Id");
   Column.fmt = LVCFMT_RIGHT;
   Itself.InsertColumn(Column.iSubItem, &Column);

   Column.cx = 200;
   Column.iSubItem = COLUMN_FILENAME;
   Column.pszText = _T("Filename");
   Column.fmt = LVCFMT_LEFT;
   Itself.InsertColumn(Column.iSubItem, &Column);

   Column.cx = 150;
   Column.iSubItem = COLUMN_WINDOW;
   Column.pszText = _T("Window");
   Column.fmt = LVCFMT_LEFT;
   Itself.InsertColumn(Column.iSubItem, &Column);

   Column.cx = 65;
   Column.iSubItem = COLUMN_WORKINGSET;
   Column.pszText = _T("RAM");
   Column.fmt = LVCFMT_RIGHT;
   Itself.InsertColumn(Column.iSubItem, &Column);

   Column.cx = 65;
   Column.iSubItem = COLUMN_HANDLECOUNT;
   Column.pszText = _T("#Handle");
   Column.fmt = LVCFMT_RIGHT;
   Itself.InsertColumn(Column.iSubItem, &Column);

   Column.cx = 100;
   Column.iSubItem = COLUMN_PARENTPROCESS;
   Column.pszText = _T("Parent Process");
   Column.fmt = LVCFMT_LEFT;
   Itself.InsertColumn(Column.iSubItem, &Column);

   // if creation is successful
   return 0;
}

int CAplView::InsertItem(LPVOID pItem, int nItem)
{
//
// Insert an item into the control. The key here is that the item's
// lParam is actually a pointer to the object that holds the item's
// data. pszText is set to LPSTR_TEXTCALLBACK so our OnGetDispInfo
// function will be called to get the text of an item or subitem.
//
   LVITEM item;
   item.iItem     = (nItem == -1)? GetListCtrl().GetItemCount() : nItem;
   item.iSubItem  = 0;
   item.mask      = LVIF_TEXT | LVIF_PARAM;
   item.pszText   = LPSTR_TEXTCALLBACK;
   item.iImage    = 0;
   item.lParam    = (LPARAM) pItem;

   return GetListCtrl().InsertItem (&item);
}

void CAplView::OnNeedText(LPVOID pItem, int nColumn, CString& szContent)
{
// set default value
   szContent = _T("");

// retrieve the CProcess* inserted with the item as lParam
   CProcess* pObject = (CProcess*)pItem;

   if (pObject != NULL)
      switch (nColumn)
	  {
         case COLUMN_PID:
         {
            DWORD dwPID = pObject->GetPID();
            if (dwPID != (DWORD)-1)
               szContent.Format(_T("%u"), dwPID);
            else
               szContent = _T("?");
         }
         break;

      // name
         case COLUMN_NAME:
            szContent = pObject->GetName();
         break;

      // filename
         case COLUMN_FILENAME:
            szContent = pObject->GetFilename();
         break;

      // Window title
         case COLUMN_WINDOW:
         {
            szContent = pObject->GetMainWindowTitle();

         // if there is no title, display the window handle in Hex
            if (szContent.GetLength() == 0)
               szContent.Format(_T("....no title for 0x%x"), pObject->GetMainWindowHandle());
         }
         break;

      // parent process ID
         case COLUMN_PARENTPROCESS:
         {
            DWORD dwPID = pObject->GetParentProcessID();
            if (dwPID != (DWORD)-1)
            {
            // get the process list from the document
               CAplDoc* pDoc = GetDocument();

            // get the process name from the process list
               CProcessList&  ProcessList = pDoc->GetProcessList();
               CProcess* pProcess = (CProcess*)ProcessList.GetProcess(dwPID);
               if (pProcess != NULL)
                  szContent.Format(_T("%s %u"), (LPCTSTR)pProcess->GetName(), dwPID);
               else
                  szContent.Format(_T("....process %u"), dwPID);
            }
            else
               szContent = _T("?");

         }
         break;

      // handle count
         case COLUMN_HANDLECOUNT:
         {
            DWORD dwHandleCount = pObject->GetHandleCount();
            if (dwHandleCount != (DWORD)-1)
               szContent.Format(_T("%u"), dwHandleCount);
            else
               szContent = _T("?");
         }
         break;

      // handle count
         case COLUMN_WORKINGSET:
         {
            DWORD dw = pObject->GetWorkingSet();
            if (dw != (DWORD)-1)
               szContent.Format(_T("%u K"), dw/1024);
            else
               szContent = _T("?");
         }
         break;
      }
}

// this handler is called when the control needs to know what to display 
// in each line and for each column
void CAplView::OnGetDispInfo(NMHDR* pNMHDR, LRESULT* pResult) 
{
   LV_DISPINFO* pDispInfo = (LV_DISPINFO*)pNMHDR;

// call virtual method OnNeedText() to get the text to be displayed in each column
   if (pDispInfo->item.mask & LVIF_TEXT)
   {
      LPVOID pItem = (LPVOID) pDispInfo->item.lParam;
      CString szDescription;
      OnNeedText (pItem, pDispInfo->item.iSubItem, szDescription);
      ::lstrcpy(pDispInfo->item.pszText, szDescription);
   }
      
   *pResult = 0;
}

void CAplView::OnItemchanged(NMHDR* pNMHDR, LRESULT* pResult) 
{
// check if selection is enabled
   if (!m_bNoSelection)
   {
      NM_LISTVIEW* pNMListview = (NM_LISTVIEW*)pNMHDR;
   
      if (pNMListview->uNewState == (LVIS_FOCUSED | LVIS_SELECTED))  // value of 3
      {
         CProcess* pProcess = (CProcess*)pNMListview->lParam;

         CAplDoc* pDoc = GetDocument();
		 pDoc->SetCurrentProcess(this, pProcess);
      }
   }

   *pResult = 0;
}

void CAplView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
// lHint = View Message
   switch(lHint)
   {
   // default MFC redrawing process
      case VM_REDRAW :
         CListView::OnUpdate(pSender, lHint, pHint);
      break;

   // the user has selected a new process (through the tree view)
      case VM_SET_CURRENT_PROCESS :
      {
      // set the selection for the listview
         CProcess* pProcess = (CProcess*)pHint;
         if (pProcess != NULL)
         {
         // change the selection in the listview
            DWORD PID = pProcess->GetPID();
            int iRow = 0;
            if (m_ProcessIndex.Lookup((LPVOID)PID, iRow))
            {
            // disable selection notification 
               m_bNoSelection = TRUE;

            // select the list item which corresponds to the process
               GetListCtrl().SetItemState(iRow, LVIS_SELECTED | LVIS_FOCUSED, LVIS_SELECTED | LVIS_FOCUSED);
               GetListCtrl().EnsureVisible(iRow, FALSE);

            // enable selection notification again
               m_bNoSelection = FALSE;
            }
            
         }
      }
      break;

   // the current font has been changed by the user
      case VM_CHANGEFONT:
      {
      // change list control font
         SendMessage(WM_SETFONT, (WPARAM)(HFONT)pHint, 0L); 

      // redraw all
         Invalidate(TRUE);
      }
      break;
   }
}

void CAplView::OnEditCopy() 
{
// get the string to be sent to the clipboard
   LPTSTR pText = GetClipboardString();
   if (pText != NULL)
   {
      CAplApp* pApp = (CAplApp*)AfxGetApp();
      if (!pApp->CopyTextToClipboard(pText))
         ::MessageBox(
            ::GetActiveWindow(),
            _T("Impossible to copy the process list to the clipboard!"),
            _T("Advanced Process List"),
            MB_OK | MB_ICONHAND
            );
   // don't forget to delete the string
      delete [] pText;
   }
   else
      ::MessageBox(
         ::GetActiveWindow(),
         _T("Nothing to copy to clipboard..."),
         _T("Advanced Process List"),
         MB_OK | MB_ICONINFORMATION
         );
}

LPTSTR CAplView::GetClipboardString()
{
   TCHAR* pText = NULL;

   CAplDoc* pDoc = GetDocument();
   ASSERT(pDoc != NULL);
   if (pDoc != NULL)
   {
   // get the process list from document
      CProcessList&  ProcessList = pDoc->GetProcessList();

   // allocate the returned buffer              
   // according to the max length of each column   3rd & 4th    v-- max len of other columns
      pText = new TCHAR[ProcessList.GetCount() * (2*_MAX_PATH + 80)];
      if (pText != NULL)
      {
         CString     szLine;
         CString     szBuffer;
         TCHAR*      pNext    = pText;
         int         iCount   = 0;
         POSITION    Pos      = 0;
         CProcess*   pProcess = NULL;

         pText[0] = _T('\0');

      // title first
         int len;
         _stprintf(pNext, _T("Name\tID\tFilename\tWindow\tRAM\t#Handles\tParent\r\n%n"),&len);
         pNext += len;

      // concat each process description
         CString szDescription;
         const TCHAR* pTAB =_T("\t"); 
         for (
               pProcess = ProcessList.GetFirst(Pos); 
               (pProcess != NULL); 
               pProcess = ProcessList.GetNext(Pos)
             )
         {
         // add each process into the buffer
            if (pProcess != NULL)
            {
               for (
                     DWORD dwColumn = COLUMN_NAME; 
                     (dwColumn <= COLUMN_PARENTPROCESS); 
                     dwColumn++
                   )
               {
                  OnNeedText(pProcess, dwColumn, szDescription);
                  _tcscpy(pNext, szDescription);
                  pNext += szDescription.GetLength();
                  if (dwColumn != COLUMN_PARENTPROCESS)
                  {
                     _tcscpy(pNext, pTAB);
                     pNext += _tcslen(pTAB);
                  }
               }

            // end of line
               _tcscpy(pNext, _T("\r\n"));
               pNext += _tcslen(_T("\r\n"));
            }
         }
      }
   }

   return pText;
}


