// Module.cpp: implementation of the CModule class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Module.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

IMPLEMENT_DYNCREATE(CModule, CObject)

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CModule::CModule()
{
}

CModule::CModule(HMODULE hModule, LPCSTR szFilename)
{
   m_hModule    = hModule;
   m_szFilename = szFilename;
}

CModule::~CModule()
{
}


//Debug assertion functions
void CModule::AssertValid() const
{
   CObject::AssertValid();

   ASSERT((m_szFilename != _T("")) && (m_hModule != 0));
}

void CModule::Dump(CDumpContext& dc) const
{
   CObject::Dump(dc);

   dc << "\nm_szFilename = " << (LPCTSTR)m_szFilename;
   dc << "\nm_hModule    = " << m_hModule;
   dc << "\n";
}


//////////////////////////////////////////////////////////////////////
// CModuleList Class
//////////////////////////////////////////////////////////////////////

#include "Psapi.h"

IMPLEMENT_DYNCREATE(CModuleList, CObject)

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CModuleList::CModuleList()
{

}

CModuleList::~CModuleList()
{

}

// fill the list with the given process
void CModuleList::Refresh(HANDLE hProcess)
{
   HMODULE  hMods[MAX_MODULES];
   DWORD    cbNeeded;

   if (::EnumProcessModules(hProcess, hMods, sizeof(hMods), &cbNeeded))
   {
      if (cbNeeded > 0)
      {
      // free the module array current content 
         FreeModuleArray();

      // set its new size
         m_ModuleArray.SetSize((cbNeeded / sizeof(HMODULE)) + 1);

         TCHAR szModName[MAX_PATH];
         for (UINT iModule = 0; iModule < (cbNeeded / sizeof(HMODULE)); iModule++)
         {
         // Get the full path to the module's file
            if (::GetModuleFileNameEx(hProcess, hMods[iModule], szModName, sizeof(szModName)))
            {
            // create a CModule object to store the module info
               CModule* pModule = new CModule(hMods[iModule], szModName);
            
            // insert it into the internal module array
               if (pModule != NULL)
                  m_ModuleArray.InsertAt(iModule, pModule);
            }
         }
      }
   }   
}

// free each object stored into the array and empty the array itself
void CModuleList::FreeModuleArray()
{
   if (m_ModuleArray.GetSize() > 0)
   {
   // free each element
      for (int iModule = 0; iModule <= m_ModuleArray.GetUpperBound(); iModule++)
         delete (CModule*) m_ModuleArray[iModule];

   // delete the array itself
      m_ModuleArray.RemoveAll();
   }
}

// return number of modules in the list
int CModuleList::GetCount()
{
   return m_ModuleArray.GetSize();
}

// get the first module to start an enumeration
CModule* CModuleList::GetFirst(int& pos)
{
   pos = 0;
   return GetNext(pos);
}

// get the next module of an enumeration
CModule* CModuleList::GetNext(int& pos)
{
   CModule* pModule = NULL;
   if (pos <= m_ModuleArray.GetUpperBound())
      pModule = (CModule*) m_ModuleArray[pos++];

   return pModule;
}
