// ParentView.cpp : implementation of the CParentView class
//

#include "stdafx.h"
#include "Apl.h"

#include "AplDoc.h"
#include "ParentView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CParentView

IMPLEMENT_DYNCREATE(CParentView, CTreeView)

BEGIN_MESSAGE_MAP(CParentView, CTreeView)
   //{{AFX_MSG_MAP(CParentView)
   ON_WM_CREATE()
   ON_WM_DESTROY()
   ON_NOTIFY_REFLECT(TVN_GETDISPINFO, OnGetDispInfo)
	ON_NOTIFY_REFLECT(TVN_SELCHANGED, OnSelchanged)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CParentView construction/destruction

CParentView::CParentView()
{
   m_bNoSelection = FALSE;
}

CParentView::~CParentView()
{
}

BOOL CParentView::PreCreateWindow(CREATESTRUCT& cs) 
{
// default processing
   BOOL bReturn = CTreeView::PreCreateWindow(cs);

// the window class must be set here because the splitter creation
// process in CreateView() does not allow us to set the class name
   cs.lpszClass = WC_TREEVIEW;   // "SysTree view32"

// set default tree styles 
   cs.style |= 
      TVS_HASLINES | 
      TVS_HASBUTTONS | 
      TVS_LINESATROOT | 
      TVS_SHOWSELALWAYS | 
      TVS_DISABLEDRAGDROP;

   return bReturn;
}


/////////////////////////////////////////////////////////////////////////////
// CParentView drawing

// called when the view will be displayed for the first time
void CParentView::OnInitialUpdate() 
{
// default MFC processing
   CTreeView::OnInitialUpdate();
   
// fill the tree
   FillParentTree();
}

void CParentView::FillParentTree()
{
// don't forget to delete the actual content of the tree view
   GetTreeCtrl().DeleteAllItems();
   m_ProcessItems.RemoveAll();

// add a fake process called "Process Tree" as common root
   TV_INSERTSTRUCT tvInsert;
   tvInsert.hParent              = TVI_ROOT;
   tvInsert.hInsertAfter         = TVI_LAST;
   tvInsert.item.mask            = TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE | TVIF_PARAM;
   tvInsert.item.iImage          = STATE_ROOT;
   tvInsert.item.iSelectedImage  = STATE_ROOT;
   tvInsert.item.pszText         = (LPTSTR)_T("Process Tree");
   tvInsert.item.lParam          = (LPARAM)-1;
   m_hRoot = GetTreeCtrl().InsertItem(&tvInsert);
 
// get the process list from the document
   CAplDoc* pDoc = GetDocument();
   ASSERT(pDoc != NULL);
   if (pDoc != NULL)
   {
      CProcessList&  ProcessList = pDoc->GetProcessList();
      CProcess*      pProcess;
      POSITION       Pos;      

      CMapPtrToPtr Children;

      // enumerate processes and try to add each into the tree 
      for (
         pProcess = ProcessList.GetFirst(Pos); 
         (pProcess != NULL); 
         pProcess = ProcessList.GetNext(Pos)
          )
      {
         if (pProcess != NULL)
         {
         // clear the children map
            Children.RemoveAll();
            InsertProcess(
               pProcess->GetPID(), 
               ProcessList, 
               m_ProcessItems,
               Children);
         }
      }
   }
}

HTREEITEM CParentView::InsertProcess(
   DWORD          PID, 
   CProcessList&  ProcessList,
   CTypedPtrMap<CMapPtrToPtr, DWORD, HTREEITEM>& InsertedProcesses,
   CMapPtrToPtr&  Children
   )
{
// check if the process ID has not been already inserted into the tree
   HTREEITEM hNode = NULL;
   if (InsertedProcesses.Lookup((LPVOID)PID, hNode))
   {
   // already inserted
      return hNode;
   }
   else
   {// not already inserted
      TV_INSERTSTRUCT tvInsert;
      tvInsert.hInsertAfter   = TVI_LAST;
      tvInsert.item.mask      = TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE | TVIF_PARAM;
   
   // The OnNeedText() virtual method will be responsible 
   // for giving the text label for a dead or running process
      tvInsert.item.pszText   = LPSTR_TEXTCALLBACK;
      tvInsert.item.lParam    = (LPARAM) PID;
   
   // determine which image to display
      CProcess*   pProcess = ProcessList.GetProcess(PID);
      if (pProcess != NULL)
      {// the process is running

      // set its image
         if (pProcess->GetChildrenCount() > 0)
         {
         // -> it is a parent process
            tvInsert.item.iImage         = STATE_PARENT;
            tvInsert.item.iSelectedImage = STATE_PARENT;
         }
         else
         {
         // -> it is a child process
            tvInsert.item.iImage         = STATE_CHILD;
            tvInsert.item.iSelectedImage = STATE_CHILD;
         }
      }
      else
      {// the process is no longer running
      
      // -> it is a parent process
         tvInsert.item.iImage         = STATE_PARENT;
         tvInsert.item.iSelectedImage = STATE_PARENT;
      }

   // is it a dead process?
      CString szName;
      if (pProcess == NULL)
      {// the process is no longer running 
      // --> impossible to know its parent so it is a root
      //     and we insert it under the tree root 
         tvInsert.hParent = m_hRoot;
      }
      else
      {// it is running
      // therefore, we have to know first where its parent is inserted
         DWORD ParentPID = pProcess->GetParentProcessID();
         DWORD PIDStored;

      // detect loops
         if (Children.Lookup((LPVOID)PID, (LPVOID&)PIDStored))
         {
         // there is a loop: this process is dead and its ID has been reused
         // --> add it as a special root
            tvInsert.hParent = m_hRoot;
            szName.Format(_T("%s or Dead process %u"), (LPCTSTR)pProcess->GetName(), PID);
            tvInsert.item.pszText = (LPTSTR)szName.GetBuffer(_MAX_PATH);
            szName.ReleaseBuffer(-1);
         }
         else
         {
         // is its parent already inserted
            HTREEITEM hParentNode = NULL;
            if (InsertedProcesses.Lookup((LPVOID)ParentPID, hParentNode))
               tvInsert.hParent = hParentNode;
            else
            {
            // first insert its parent before itself
               Children[(LPVOID)PID] = (LPVOID)PID;
               tvInsert.hParent = 
                  InsertProcess(
                     ParentPID, 
                     ProcessList, 
                     InsertedProcesses, 
                     Children);
               Children.RemoveKey((LPVOID)PID);
            }
         }
      }
      
   // insert it into the tree and the cross-reference map 
   // if not already done during the parent insertion
      if (!InsertedProcesses.Lookup((LPVOID)PID, hNode))
      {
         hNode = GetTreeCtrl().InsertItem(&tvInsert);
         InsertedProcesses[(LPVOID)PID] = hNode;
      }

      return hNode;
   }
}

/////////////////////////////////////////////////////////////////////////////
// CParentView diagnostics

#ifdef _DEBUG
void CParentView::AssertValid() const
{
   CTreeView::AssertValid();
}

void CParentView::Dump(CDumpContext& dc) const
{
   CTreeView::Dump(dc);
}

CAplDoc* CParentView::GetDocument() // non-debug version is inline
{
   ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CAplDoc)));
   return (CAplDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CParentView message handlers

int CParentView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
// default MFC processing
   if (CTreeView::OnCreate(lpCreateStruct) == -1)
      return -1;
   
   CBitmap  bitmap;  // DeleteObject() will be called in its destructor
   TRACE("CParentView::OnCreate Entered\n");

// load image list from resources
   if (bitmap.LoadBitmap(IDB_PARENT_IMAGE_LIST))
   {
      BITMAP bmInfo;
      VERIFY(bitmap.GetObject(sizeof(bmInfo), &bmInfo) != 0);

   // Create the parent/child image lists.
      if (
            m_TreeImages.Create(
               PICTURE_WIDTH,                   // each small bitmap width
               bmInfo.bmHeight,                 // bitmap height
               ILC_MASK,                        // transparency mask management
               bmInfo.bmWidth / PICTURE_WIDTH,  // number of images within the image list
               1                                // ...
               )
         )
      {
      // set the bitmap in the image list with the masking color (= light gray)
         if (m_TreeImages.Add(&bitmap, RGB(192, 192, 192)) != -1)
         {
         // associate the image list to the tree view
            GetTreeCtrl().SetImageList(&m_TreeImages, TVSIL_NORMAL);

         // creation is successful
            return 0;
         }
         else
            TRACE("Impossible to add the bitmap to the image list\n");
      }
      else
         TRACE("Impossible to create image list\n");
   }
   else
      TRACE("Impossible to load bitmap\n");


// creation has failed
   return -1;
}

void CParentView::OnDestroy() 
{
// don't forget to delete the associated image list
   m_TreeImages.DeleteImageList();

// default MFC processing
   CTreeView::OnDestroy();
}

void CParentView::OnGetDispInfo(NMHDR* pNMHDR, LRESULT* pResult) 
{
   TV_DISPINFO* pTVDispInfo = (TV_DISPINFO*)pNMHDR;
   
// call virtual method OnNeedText() to get the text to be displayed in each column
   if (pTVDispInfo->item.mask & LVIF_TEXT)
   {
      CString  szLabel;
      LPVOID   pItem = (LPVOID) pTVDispInfo->item.lParam;
      
   // call the virtual method to get the text to be displayed for a tree item
      OnNeedText(pItem, szLabel);
      
      ::lstrcpy(pTVDispInfo->item.pszText, szLabel);
   }
      
   *pResult = 0;
}

// ask for the text of the tree item corresponding to pItem 
void CParentView::OnNeedText(LPVOID pItem, CString& szText)
{
// default value
   szText = _T("");

// the process ID is associated to each tree item as lParam
   DWORD PID = (DWORD)pItem;

// get the process list from the document
   CAplDoc* pDoc = GetDocument();
   ASSERT(pDoc != NULL);
   if (pDoc != NULL)
   {
      CProcessList&  ProcessList = pDoc->GetProcessList();

   // build the tree item text label
      CString     szBuffer;
      CProcess*   pProcess = ProcessList.GetProcess(PID);
      if (pProcess != NULL)
      {
      // the process is running
         szBuffer = pProcess->GetName();
         szText.Format(_T("%s %u"), (LPCTSTR)szBuffer, PID);
      }
      else
      {
      // the process is no more running
         szText.Format(_T("process %u"), PID);
      }
   }
}

void CParentView::OnSelchanged(NMHDR* pNMHDR, LRESULT* pResult)
{
// check if selection is unabled
   if (!m_bNoSelection)
   {
      NM_TREEVIEW* pNMTreeview = (NM_TREEVIEW*)pNMHDR;
      ASSERT(pNMTreeview != NULL);
  
      DWORD PID = (DWORD)pNMTreeview->itemNew.lParam;

      CAplDoc* pDoc = GetDocument();
      ASSERT(pDoc != NULL);
      if (pDoc != NULL)
      {
         CProcessList& ProcessList = pDoc->GetProcessList();
         CProcess* pProcess = ProcessList.GetProcess(PID);

         pDoc->SetCurrentProcess(this, pProcess);
      }
   }

   *pResult = 0;
}

void CParentView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
// lHint = View Message
   switch(lHint)
   {
   // default MFC redrawing process
      case VM_REDRAW :
         CTreeView::OnUpdate(pSender, lHint, pHint);
      break;

   // the user has selected a new process
      case VM_SET_CURRENT_PROCESS :
      {
         CProcess* pProcess = (CProcess*)pHint;
         if (pProcess != NULL)
         {
         // change the selection in the tree view
            DWORD PID = pProcess->GetPID();
            HTREEITEM hNode = NULL;
            if (m_ProcessItems.Lookup((LPVOID)PID, hNode))
            {
            // disable selection notification
               m_bNoSelection = TRUE;
               
            // select the tree item which corresponds to the process
               if (GetTreeCtrl().SelectItem(hNode))
                  GetTreeCtrl().EnsureVisible(hNode);
               
            // enable selection notification again
               m_bNoSelection = FALSE;
            }
            
         }
      }
      break;

   // the current font has been changed by the user
      case VM_CHANGEFONT:
      {
      // change list control font
         SendMessage(WM_SETFONT, (WPARAM)(HFONT)pHint, 0L); 

      // redraw all
         Invalidate(TRUE);
      }
      break;

   }
}

